@@ -58,8 +58,8 @@ print.bal.tab <- function(x, disp.m.threshold = "as.is", disp.v.threshold = "as.
     if (!is.na(match("bal.tab.cont", class(x)))) {
         keep <- as.logical(c(TRUE, 
                              p.ops$un, 
-                             p.ops$disp.adj, 
-                             !is.null(p.ops$r.threshold)))
+                             rep(c(p.ops$disp.adj, 
+                             !is.null(p.ops$r.threshold)), p.ops$nweights + !p.ops$disp.adj)))
     }
     else {
         keep <- as.logical(c(TRUE, 
@@ -67,52 +67,67 @@ print.bal.tab <- function(x, disp.m.threshold = "as.is", disp.v.threshold = "as.
                              p.ops$un*p.ops$disp.means, 
                              p.ops$un, 
                              p.ops$un*p.ops$disp.v.ratio, 
-                             p.ops$disp.adj*p.ops$disp.means, 
+                             rep(c(p.ops$disp.adj*p.ops$disp.means, 
                              p.ops$disp.adj*p.ops$disp.means, 
                              p.ops$disp.adj, 
                              !is.null(p.ops$m.threshold), 
                              p.ops$disp.adj*p.ops$disp.v.ratio, 
-                             !is.null(p.ops$v.threshold)))
+                             !is.null(p.ops$v.threshold)), p.ops$nweights + !p.ops$disp.adj)))
     }
     
     if (!is.null(call)) {
         cat("\nCall:", deparse(call), sep = "\n")
+        cat("\n")
     }
     
-    cat("\nBalance Measures:\n")
+    cat("Balance Measures:\n")
     print.data.frame(replaceNA(round_df(balance[, keep], digits)))
+    cat("\n")
     
     if (!is.null(baltal.r)) {
-        cat("\nBalance tally for correlations:\n")
+        cat("Balance tally for correlations:\n")
         print.data.frame(x$Balanced.Corr)
+        cat("\n")
     }
     if (!is.null(maximbal.r)) {
-        cat("\nVariable with the greatest treatment correlation:\n")
+        cat("Variable with the greatest treatment correlation:\n")
         print.data.frame(round_df(x$Max.Imbalance.Corr, digits))
+        cat("\n")
     }
     if (!is.null(baltal.m)) {
-        cat("\nBalance tally for mean differences:\n")
+        cat("Balance tally for mean differences:\n")
         print.data.frame(x$Balanced.Means)
+        cat("\n")
     }
     if (!is.null(maximbal.m)) {
-        cat("\nVariable with the greatest mean difference:\n")
+        cat("Variable with the greatest mean difference:\n")
         print.data.frame(round_df(x$Max.Imbalance.Means, digits))
+        cat("\n")
     }
     if (!is.null(baltal.v)) {
-        cat("\nBalance tally for variance ratios:\n")
+        cat("Balance tally for variance ratios:\n")
         print.data.frame(x$Balanced.Variances, digits)
+        cat("\n")
     }
     if (!is.null(maximbal.v)) {
-        cat("\nVariable with the greatest variance ratio:\n")
+        cat("Variable with the greatest variance ratio:\n")
         print.data.frame(round_df(x$Max.Imbalance.Variances, digits))
+        cat("\n")
     }
     if (!is.null(nn)) {
         for (i in rownames(x$Observations)) {
             if (all(x$Observations[i,] == 0)) x$Observations <- x$Observations[rownames(x$Observations)!=i,]
         }
         if (!is.na(x$Observations["Matched (Unweighted)",]) && all(x$Observations["Matched",] == x$Observations["Matched (Unweighted)",])) x$Observations <- x$Observations[rownames(x$Observations)!="Matched (Unweighted)",]
-        cat(paste0("\n", attr(x$Observations, "tag"), ":\n"))
+        cat(paste0(attr(x$Observations, "tag"), ":\n"))
+        print.warning <- FALSE
+        if (length(attr(x$Observations, "ss.type")) > 1 && length(unique(attr(x$Observations, "ss.type")[-1])) > 1) {
+            ess <- ifelse(attr(x$Observations, "ss.type") == "ess", "*", "")
+            x$Observations <- setNames(cbind(x$Observations, ess), c(names(x$Observations), ""))
+            print.warning <- TRUE
+        }
         print.data.frame(replaceNA(x$Observations), digits = digits)
+        if (print.warning) cat("* indicates effective sample size")
     }
     invisible(x)
 }
@@ -169,7 +184,8 @@ print.bal.tab.subclass <- function(x, disp.m.threshold = "as.is", disp.v.thresho
     }
     
     if (!is.null(call)) {
-        cat("\nCall:", deparse(call), sep = "\n")
+        cat("Call:", deparse(call), sep = "\n")
+        cat("\n")
     }
     
     if (p.ops$disp.subclass) {
@@ -180,11 +196,12 @@ print.bal.tab.subclass <- function(x, disp.m.threshold = "as.is", disp.v.thresho
                                !is.null(p.ops$m.threshold), 
                                p.ops$disp.adj * p.ops$disp.v.ratio, 
                                !is.null(p.ops$v.threshold)))
-        cat("\nBalance by subclass:")
+        cat("Balance by subclass:")
         for (i in names(s.balance)) {
             cat(paste0("\n - - - Subclass ", i, " - - - \n"))
             print.data.frame(replaceNA(round_df(s.balance[[i]][, s.keep], digits)))
         }
+        cat("\n")
     }
     
     a.s.keep <- as.logical(c(TRUE, 
@@ -195,28 +212,33 @@ print.bal.tab.subclass <- function(x, disp.m.threshold = "as.is", disp.v.thresho
                              p.ops$disp.adj*p.ops$disp.means, 
                              p.ops$disp.adj, 
                              !is.null(p.ops$m.threshold)))
-    cat("\nBalance measures across subclasses:\n")
+    cat("Balance measures across subclasses:\n")
     print.data.frame(replaceNA(round_df(b.a.subclass[, a.s.keep], digits)))
+    cat("\n")
     
     if (!is.null(baltal.m.subclass)) {
-        cat("\nBalance tally for mean differences across subclasses:\n")
+        cat("Balance tally for mean differences across subclasses:\n")
         print.data.frame(baltal.m.subclass)
+        cat("\n")
     }
     if (!is.null(maximbal.m.subclass)) {
-        cat("\nVariable with the greatest mean difference across subclasses:\n")
+        cat("Variable with the greatest mean difference across subclasses:\n")
         print.data.frame(round_df(maximbal.m.subclass, digits))
+        cat("\n")
     }
     if (!is.null(baltal.v.subclass)) {
-        cat("\nBalance tally for variance ratios across subclasses:\n")
+        cat("Balance tally for variance ratios across subclasses:\n")
         print.data.frame(baltal.v.subclass)
+        cat("\n")
     }
     if (!is.null(maximbal.v.subclass)) {
-        cat("\nVariable with the greatest variance ratios across subclasses:\n")
+        cat("Variable with the greatest variance ratios across subclasses:\n")
         print.data.frame(round_df(maximbal.v.subclass, digits))
+        cat("\n")
     }
     
     if (!is.null(s.nn)) {
-        cat(paste0("\n", attr(x$Subclass.Observations, "tag"), ":\n"))
+        cat(paste0(attr(x$Subclass.Observations, "tag"), ":\n"))
         print.data.frame(replaceNA(x$Subclass.Observations), digits = digits)
     }
     
@@ -323,8 +345,8 @@ print.bal.tab.cluster <- function(x, disp.m.threshold = "as.is", disp.v.threshol
     if (!is.na(match("bal.tab.cont.cluster", class(x)))) {
         keep <- as.logical(c(TRUE, 
                              p.ops$un, 
-                             p.ops$disp.adj, 
-                             !is.null(p.ops$r.threshold)))
+                             rep(c(p.ops$disp.adj, 
+                             !is.null(p.ops$r.threshold)), p.ops$nweights)))
     }
     else {
         keep <- as.logical(c(TRUE, 
@@ -332,16 +354,18 @@ print.bal.tab.cluster <- function(x, disp.m.threshold = "as.is", disp.v.threshol
                              p.ops$un*p.ops$disp.means, 
                              p.ops$un, 
                              p.ops$un*p.ops$disp.v.ratio, 
-                             p.ops$disp.adj*p.ops$disp.means, 
+                             rep(c(p.ops$disp.adj*p.ops$disp.means, 
                              p.ops$disp.adj*p.ops$disp.means, 
                              p.ops$disp.adj, 
                              !is.null(p.ops$m.threshold), 
                              p.ops$disp.adj*p.ops$disp.v.ratio, 
-                             !is.null(p.ops$v.threshold)))
+                             !is.null(p.ops$v.threshold)), p.ops$nweights + !p.ops$disp.adj)))
     }
     
+    #Printing
     if (!is.null(call)) {
-        cat("\nCall:", deparse(call), sep = "\n")
+        cat("Call:", deparse(call), sep = "\n")
+        cat("\n")
     }
     
     if (length(which.cluster)>0) {
@@ -356,9 +380,18 @@ print.bal.tab.cluster <- function(x, disp.m.threshold = "as.is", disp.v.threshol
             }
             if (!is.na(c.balance[[i]][["Observations"]]["Matched (Unweighted)",]) && all(c.balance[[i]][["Observations"]]["Matched",] == c.balance[[i]][["Observations"]]["Matched (Unweighted)",])) c.balance[[i]][["Observations"]] <- c.balance[[i]][["Observations"]][rownames(c.balance[[i]][["Observations"]])!="Matched (Unweighted)",]
             cat(paste0("\n", attr(c.balance[[i]][["Observations"]], "tag"), ":\n"))
+            print.warning <- FALSE
+            if (length(attr(c.balance[[i]][["Observations"]], "ss.type")) > 1 && length(unique(attr(c.balance[[i]][["Observations"]], "ss.type")[-1])) > 1) {
+                ess <- ifelse(attr(c.balance[[i]][["Observations"]], "ss.type") == "ess", "*", "")
+                xc.balance[[i]][["Observations"]] <- setNames(cbind(c.balance[[i]][["Observations"]], ess), c(names(c.balance[[i]][["Observations"]]), ""))
+                print.warning <- TRUE
+            }
             print.data.frame(replaceNA(round_df(c.balance[[i]][["Observations"]], digits)))
+            if (print.warning) cat("* indicates effective sample size")
+            
         }
         cat(paste0(paste(rep(" -", round(nchar(paste0("\n - - - Cluster: ", names(c.balance)[i], " - - - "))/2)), collapse = ""), " \n"))
+        cat("\n")
     }
     
     if (isTRUE(as.logical(p.ops$cluster.summary))) {
@@ -370,10 +403,10 @@ print.bal.tab.cluster <- function(x, disp.m.threshold = "as.is", disp.v.threshol
                                    p.ops$un*CF["mean"],
                                    p.ops$un*CF["median"],
                                    p.ops$un*CF["max"],
-                                   p.ops$disp.adj*CF["min"],
+                                   rep(c(p.ops$disp.adj*CF["min"],
                                    p.ops$disp.adj*CF["mean"],
                                    p.ops$disp.adj*CF["median"],
-                                   p.ops$disp.adj*CF["max"]))
+                                   p.ops$disp.adj*CF["max"]), p.ops$nweights + !p.ops$disp.adj)))
         }
         else {
             s.keep <- as.logical(c(TRUE, 
@@ -385,25 +418,33 @@ print.bal.tab.cluster <- function(x, disp.m.threshold = "as.is", disp.v.threshol
                                    p.ops$un*p.ops$disp.v.ratio*CF["mean"],
                                    p.ops$un*p.ops$disp.v.ratio*CF["median"],
                                    p.ops$un*p.ops$disp.v.ratio*CF["max"],
-                                   p.ops$disp.adj*CF["min"],
+                                   rep(c(p.ops$disp.adj*CF["min"],
                                    p.ops$disp.adj*CF["mean"],
                                    p.ops$disp.adj*CF["median"],
                                    p.ops$disp.adj*CF["max"],
                                    p.ops$disp.adj*p.ops$disp.v.ratio*CF["min"],
                                    p.ops$disp.adj*p.ops$disp.v.ratio*CF["mean"],
                                    p.ops$disp.adj*p.ops$disp.v.ratio*CF["median"],
-                                   p.ops$disp.adj*p.ops$disp.v.ratio*CF["max"]))
+                                   p.ops$disp.adj*p.ops$disp.v.ratio*CF["max"]), p.ops$nweights + !p.ops$disp.adj)))
         }
-        cat("\nBalance summary across all clusters:\n")
+        cat("Balance summary across all clusters:\n")
         print.data.frame(replaceNA(round_df(c.balance.summary[, s.keep], digits)))
+        cat("\n")
         
         if (!is.null(nn)) {
             for (i in rownames(x$Observations)) {
                 if (all(x$Observations[i,] == 0)) x$Observations <- x$Observations[rownames(x$Observations)!=i,]
             }
             if (!is.na(x$Observations["Matched (Unweighted)",]) && all(x$Observations["Matched",] == x$Observations["Matched (Unweighted)",])) x$Observations <- x$Observations[rownames(x$Observations)!="Matched (Unweighted)",]
-            cat(paste0("\n", attr(x$Observations, "tag"), ":\n"))
+            cat(paste0(attr(x$Observations, "tag"), ":\n"))
+            print.warning <- FALSE
+            if (length(attr(x$Observations, "ss.type")) > 1 && length(unique(attr(x$Observations, "ss.type")[-1])) > 1) {
+                ess <- ifelse(attr(x$Observations, "ss.type") == "ess", "*", "")
+                x$Observations <- setNames(cbind(x$Observations, ess), c(names(x$Observations), ""))
+                print.warning <- TRUE
+            }
             print.data.frame(replaceNA(x$Observations), digits = digits)
+            if (print.warning) cat("* indicates effective sample size")
         }
     }
 
@@ -503,7 +544,7 @@ print.bal.tab.imp <- function(x, disp.m.threshold = "as.is", disp.v.threshold =
     
     #Printing output
     if (!is.null(call)) {
-        cat("\nCall:", deparse(call), sep = "\n")
+        cat("Call:\n", deparse(call), "\n")
     }
     
     if (length(which.imp) > 0) {
@@ -513,6 +554,7 @@ print.bal.tab.imp <- function(x, disp.m.threshold = "as.is", disp.v.threshold =
             do.call(print, c(list(i.balance[[i]]), args))
         }
         cat(paste0(paste(rep(" -", round(nchar(paste0("\n - - - Imputation: ", names(i.balance)[i], " - - - "))/2)), collapse = ""), " \n"))
+        cat("\n")
     }
     
     if (isTRUE(as.logical(p.ops$imp.summary))) {
@@ -524,10 +566,10 @@ print.bal.tab.imp <- function(x, disp.m.threshold = "as.is", disp.v.threshold =
                                    p.ops$un*IF["mean"],
                                    p.ops$un*IF["median"],
                                    p.ops$un*IF["max"],
-                                   p.ops$disp.adj*IF["min"],
+                                   rep(c(p.ops$disp.adj*IF["min"],
                                    p.ops$disp.adj*IF["mean"],
                                    p.ops$disp.adj*IF["median"],
-                                   p.ops$disp.adj*IF["max"]))
+                                   p.ops$disp.adj*IF["max"]), p.ops$nweights + !p.ops$disp.adj)))
         }
         else { #binary
             s.keep <- as.logical(c(TRUE, 
@@ -539,27 +581,35 @@ print.bal.tab.imp <- function(x, disp.m.threshold = "as.is", disp.v.threshold =
                                    p.ops$un*p.ops$disp.v.ratio*IF["mean"],
                                    p.ops$un*p.ops$disp.v.ratio*IF["median"],
                                    p.ops$un*p.ops$disp.v.ratio*IF["max"],
-                                   p.ops$disp.adj*IF["min"],
+                                   rep(c(p.ops$disp.adj*IF["min"],
                                    p.ops$disp.adj*IF["mean"],
                                    p.ops$disp.adj*IF["median"],
                                    p.ops$disp.adj*IF["max"],
                                    p.ops$disp.adj*p.ops$disp.v.ratio*IF["min"],
                                    p.ops$disp.adj*p.ops$disp.v.ratio*IF["mean"],
                                    p.ops$disp.adj*p.ops$disp.v.ratio*IF["median"],
-                                   p.ops$disp.adj*p.ops$disp.v.ratio*IF["max"]))
+                                   p.ops$disp.adj*p.ops$disp.v.ratio*IF["max"]), p.ops$nweights + !p.ops$disp.adj)))
         }
         
         
-        cat("\nBalance summary across all imputations:\n")
+        cat("Balance summary across all imputations:\n")
         print.data.frame(replaceNA(round_df(i.balance.summary[, s.keep], digits)))
+        cat("\n")
         
         if (!is.null(nn)) {
             for (i in rownames(x$Observations)) {
                 if (all(x$Observations[i,] == 0)) x$Observations <- x$Observations[rownames(x$Observations)!=i,]
             }
             if (!is.na(x$Observations["Matched (Unweighted)",]) && all(x$Observations["Matched",] == x$Observations["Matched (Unweighted)",])) x$Observations <- x$Observations[rownames(x$Observations)!="Matched (Unweighted)",]
-            cat(paste0("\n", attr(x$Observations, "tag"), ":\n"))
+            cat(paste0(attr(x$Observations, "tag"), ":\n"))
+            print.warning <- FALSE
+            if (length(attr(x$Observations, "ss.type")) > 1 && length(unique(attr(x$Observations, "ss.type")[-1])) > 1) {
+                ess <- ifelse(attr(x$Observations, "ss.type") == "ess", "*", "")
+                x$Observations <- setNames(cbind(x$Observations, ess), c(names(x$Observations), ""))
+                print.warning <- TRUE
+            }
             print.data.frame(replaceNA(x$Observations), digits = digits)
+            if (print.warning) cat("* indicates effective sample size")
         }
     }
 
@@ -710,7 +760,7 @@ print.bal.tab.imp.cluster <- function(x, disp.m.threshold = "as.is", disp.v.thre
     
     #Printing output
     if (!is.null(call)) {
-        cat("\nCall:", deparse(call), sep = "\n")
+        cat("Call:\n", deparse(call), "\n")
     }
     
     if (length(which.imp) > 0) {
@@ -720,6 +770,7 @@ print.bal.tab.imp.cluster <- function(x, disp.m.threshold = "as.is", disp.v.thre
             do.call(print, c(list(i.balance[[i]]), args))
         }
         cat(paste0(paste(rep(" -", round(nchar(paste0("\n - - - - Imputation: ", names(i.balance)[i], " - - - - "))/2)), collapse = ""), " \n"))
+        cat("\n")
     }
     
     if (isTRUE(as.logical(p.ops$imp.summary))) {
@@ -731,10 +782,10 @@ print.bal.tab.imp.cluster <- function(x, disp.m.threshold = "as.is", disp.v.thre
                                    p.ops$un*IF["mean"],
                                    p.ops$un*IF["median"],
                                    p.ops$un*IF["max"],
-                                   p.ops$disp.adj*IF["min"],
+                                   rep(c(p.ops$disp.adj*IF["min"],
                                    p.ops$disp.adj*IF["mean"],
                                    p.ops$disp.adj*IF["median"],
-                                   p.ops$disp.adj*IF["max"]))
+                                   p.ops$disp.adj*IF["max"]), p.ops$nweights + !p.ops$disp.adj)))
         }
         else {
             s.keep <- as.logical(c(TRUE, 
@@ -746,19 +797,19 @@ print.bal.tab.imp.cluster <- function(x, disp.m.threshold = "as.is", disp.v.thre
                                    p.ops$un*p.ops$disp.v.ratio*IF["mean"],
                                    p.ops$un*p.ops$disp.v.ratio*IF["median"],
                                    p.ops$un*p.ops$disp.v.ratio*IF["max"],
-                                   p.ops$disp.adj*IF["min"],
+                                   rep(c(p.ops$disp.adj*IF["min"],
                                    p.ops$disp.adj*IF["mean"],
                                    p.ops$disp.adj*IF["median"],
                                    p.ops$disp.adj*IF["max"],
                                    p.ops$disp.adj*p.ops$disp.v.ratio*IF["min"],
                                    p.ops$disp.adj*p.ops$disp.v.ratio*IF["mean"],
                                    p.ops$disp.adj*p.ops$disp.v.ratio*IF["median"],
-                                   p.ops$disp.adj*p.ops$disp.v.ratio*IF["max"]))
+                                   p.ops$disp.adj*p.ops$disp.v.ratio*IF["max"]), p.ops$nweights + !p.ops$disp.adj)))
         }
         
         
         if (length(which.cluster) > 0) {
-            cat("\nCluster balance summary across all imputations:\n")
+            cat("Cluster balance summary across all imputations:\n")
             for (c in which.cluster) {
                 cat(paste0("\n - - - Cluster: ", names(i.balance.c.summary)[c], " - - - \n"))
                 cat("Balance summary across imputations:\n")
@@ -769,19 +820,35 @@ print.bal.tab.imp.cluster <- function(x, disp.m.threshold = "as.is", disp.v.thre
                 }
                 if (!is.na(i.balance.c.summary[[c]][["Cluster.Observations"]]["Matched (Unweighted)",]) && all(i.balance.c.summary[[c]][["Cluster.Observations"]]["Matched",] == i.balance.c.summary[[c]][["Cluster.Observations"]]["Matched (Unweighted)",])) i.balance.c.summary[[c]][["Cluster.Observations"]] <- i.balance.c.summary[[c]][["Cluster.Observations"]][rownames(i.balance.c.summary[[c]][["Cluster.Observations"]])!="Matched (Unweighted)",]
                 cat(paste0("\n", attr(i.balance.c.summary[[c]][["Cluster.Observations"]], "tag"), ":\n"))
+                print.warning <- FALSE
+                if (length(attr(i.balance.c.summary[[c]][["Cluster.Observations"]], "ss.type")) > 1 && length(unique(attr(i.balance.c.summary[[c]][["Cluster.Observations"]], "ss.type")[-1])) > 1) {
+                    ess <- ifelse(attr(i.balance.c.summary[[c]][["Cluster.Observations"]], "ss.type") == "ess", "*", "")
+                    i.balance.c.summary[[c]][["Cluster.Observations"]] <- setNames(cbind(i.balance.c.summary[[c]][["Cluster.Observations"]], ess), c(names(i.balance.c.summary[[c]][["Cluster.Observations"]]), ""))
+                    print.warning <- TRUE
+                }
                 print.data.frame(replaceNA(round_df(i.balance.c.summary[[c]][["Cluster.Observations"]], digits)))
+                if (print.warning) cat("* indicates effective sample size")
             }
+            cat("\n")
         }
-        cat("\nBalance summary across all imputations and clusters:\n")
+        cat("Balance summary across all imputations and clusters:\n")
         print.data.frame(replaceNA(round_df(i.balance.summary[, s.keep], digits)))
+        cat("\n")
         
         if (!is.null(nn)) {
             for (i in rownames(x$Observations)) {
                 if (all(x$Observations[i,] == 0)) x$Observations <- x$Observations[rownames(x$Observations)!=i,]
             }
             if (!is.na(x$Observations["Matched (Unweighted)",]) && all(x$Observations["Matched",] == x$Observations["Matched (Unweighted)",])) x$Observations <- x$Observations[rownames(x$Observations)!="Matched (Unweighted)",]
-            cat(paste0("\n", attr(nn, "tag"), ":\n"))
-            print.data.frame(replaceNA(nn), digits = digits)
+            cat(paste0(attr(x$Observations, "tag"), ":\n"))
+            print.warning <- FALSE
+            if (length(attr(x$Observations, "ss.type")) > 1 && length(unique(attr(x$Observations, "ss.type")[-1])) > 1) {
+                ess <- ifelse(attr(x$Observations, "ss.type") == "ess", "*", "")
+                x$Observations <- setNames(cbind(x$Observations, ess), c(names(x$Observations), ""))
+                print.warning <- TRUE
+            }
+            print.data.frame(replaceNA(x$Observations), digits = digits)
+            if (print.warning) cat("* indicates effective sample size")
         }
     }
     invisible(x)
